C   PROGRAM TO COMPILE SUMMARY STATISTICS ON CONCENTRATION OUTPUT
C   FILES FROM MODFLOW-GWT ---ZONECONC Version 1.50
C   NAME=   ZONECONCv1.5.f
C   By Philip T. Harte,
C   REVISED 10/20/05
C   **************************************************************
C   CODES: ANAME= NAME OF CONCENTRATION FILE; ZNAME=NAME OF
C          ZONE FILE; NAME=EXTERNAL FILENAME WITHIN ZONE FILE
C          MNAME=NAME OF GWT FILE (READS SUBGRID LAY,ROW,COL)
C          ARRAY=CONCENTRATION ARRAY
C          IZONE=ZONE ARRAY
C          DNAME=DISCRETIZATION FILE NAME
C          NZDIM=MAX. number of zones
C          NODDIM=MAX. # OF MULTIPLE OF MODELS 'LAYER*ROW*COL'
C          NZONES=HIGHEST NUMBERED ZONE
C          NROW,NCOL,NLAY;  FLOW MODEL DIMENSIONS
C          MNROW,MNCOL,MNLAY;  SUBGRID TRANSPORT DIMENSIONS
C          JCOL,IROW,LLAYER=START COL,ROW,LAYER FOR SUBGRID
C          FROW,FCOL,FLAY=FINAL COL,ROW, LAYER FOR SUBGRID
C          INZN1=ZONE FILE UNIT #,INZN2=ZONEFILE2,INZN3=CNCA FILE
C          INZN4=MOC FILE,INZN5=TEMP. OUTPUT FILE,INZN6=DIS. FILE
C          J,I,L= denotes model grid (zone) array location
C          JJ,II,LL= denotes concentration array location
C          J,I,K=zone grid array location in zonebudget
C********************************************************************
C     SPECIFICATIONS:
      CHARACTER*20 ANAME,ONAME,HEADER,ZNAME,MNAME,CNAME,DNAME
      CHARACTER*1 C
      INTEGER NROW,NCOL,NLAY,MNROW,MNCOL,MNLAY,LLAYER,FROW,FCOL,FLAY
      PARAMETER (NODDIM=3000000,NTRDIM=12,NZDIM=25,MXZWCZ=10)
C
C      INCREASE PRECISION OF ARRAY
C       REAL*16 ARRAY,CMIN
      DOUBLE PRECISION CMIN
      DOUBLE PRECISION ARRAY
      DOUBLE PRECISION CDIFF
      DOUBLE PRECISION CAVG
      DOUBLE PRECISION CTOTAL
      DOUBLE PRECISION CMAX
      DOUBLE PRECISION CSTD
      DOUBLE PRECISION SUMTCH2
C
       LOGICAL FILEFOUND
C
       COMMON /ARRAY/IZONE
C
C        DIMENSION TRANSPORT GRID AND FLOW MODEL GRID
C
        DIMENSION ARRAY(NODDIM)
        DIMENSION IZONE(NODDIM)
      DIMENSION CMAX(NZDIM)
      DIMENSION CMIN(NZDIM)
      DIMENSION CTOTAL(NZDIM)
      DIMENSION NCOUNT(NZDIM)
      DIMENSION NCOUNTP(NZDIM)
      DIMENSION CDIFF(NZDIM)
      DIMENSION CSTD(NZDIM)
      DIMENSION CAVG(NZDIM)
      DIMENSION IMAX(NZDIM)
      DIMENSION JMAX(NZDIM)
      DIMENSION LMAX(NZDIM)
      DIMENSION IMIN(NZDIM)
      DIMENSION JMIN(NZDIM)
      DIMENSION LMIN(NZDIM)
C
C          INITIALIZE VARIABLES
C
      IERR=0
      ICOUNT=0
      IICOUNT=0
      SUMTCH2=0.
      NZONES=0
      IOUT=13
      INZN1=19
      INZN2=43
      INZN3=15
      INZN4=16
      INZN5=17
      INZN6=18
C
C---------------------------------------------------------
C                  READ INPUT/OUTPUT FILES
C
      WRITE(*,*)'ZONECONC VERSION 1.50'
      WRITE(*,*)'PROGRAM TO READ MODFLOW-GWT ASCII CONCENTRATION FILE'
      WRITE(*,*)'(CNCA) AND TABULATE STATISTICAL SUMMARIES OF '
      WRITE(*,*)'CONCENTRATIONS PER ZONE AND LAYER OF A MODEL'
      WRITE(*,*)
C
C            CNCA FILE PROMPT
C
            GO TO 9
  718 WRITE(*,*)'  PROBLEM WITH CNCA FILE, TRY AGAIN! '
C
    9 WRITE (*,*)'  ENTER INPUT CNCA FILE NAME :'
      READ(*,10) ANAME
  10  FORMAT(A20)
C
      INQUIRE(FILE=ANAME,EXIST=FILEFOUND)
      IF(.NOT.FILEFOUND) THEN
            WRITE(*,*)'CAN NOT FIND FILE: ',ANAME
            ANAME=''
            GO TO 9
            ENDIF
C
      WRITE(*,*) ' INPUT NAME IS: ', ANAME
      WRITE(*,*)
C
       OPEN(UNIT=INZN3,FILE=ANAME,STATUS='OLD',ERR=718,FORM='FORMATTED')
C
C          OUTPUT FILE PROMPT
C
            GO TO 741
  719  WRITE(*,*)' OUTPUT FILE ALREADY EXISTS, TRY AGAIN! '
  741     WRITE (*,*)'  ENTER OUTPUT FILE NAME 1:'
       READ(*,10) ONAME
C
            IF(ONAME.EQ.' ') THEN
            GO TO 741
            ENDIF
C
       OPEN(UNIT=IOUT,FILE=ONAME,STATUS='NEW',ERR=719,FORM='FORMATTED')
C
       WRITE(IOUT,*)'ZONECONC VERSION 1.50'
       WRITE(IOUT,*)'SUMMARY STATS PER LAYER AND ZONE, RUN= ',ANAME
       WRITE(IOUT,*)'OUTPUT FILE NAME = ',ONAME
C
C       GWT FILE PROMPT
C
            GO TO 721
  720   WRITE(*,*)' PROBLEM FINDING GWT FILE, TRY AGAIN! '
  721   WRITE(*,*)'  ENTER INPUT GWT FILE NAME :'
      READ(*,20) MNAME
C
      INQUIRE(FILE=MNAME,EXIST=FILEFOUND)
      IF(.NOT.FILEFOUND) THEN
            WRITE(*,*)'CAN NOT FIND FILE: ',MNAME
            MNAME=''
            GO TO 721
            ENDIF
C
      WRITE(*,*) ' INPUT GWT NAME IS: ', MNAME
C
   20 FORMAT(A20)
      OPEN(UNIT=INZN4,FILE=MNAME,STATUS='OLD',ERR=720,FORM='FORMATTED')
      READ(INZN4,*)
      READ(INZN4,*)
      READ(INZN4,*) LLAYER,FLAY,IROW,FROW,JCOL,FCOL
      WRITE(*,*) LLAYER,FLAY,IROW,FROW,JCOL,FCOL
C
      MNROW=(FROW-IROW)+1
      MNCOL=(FCOL-JCOL)+1
      MNLAY=(FLAY-LLAYER)+1
      WRITE(*,*)' SUBGRID ROWS=',MNROW,',SUBGRID COLUMNS=',MNCOL
      WRITE(*,*)' SUBGRID LAYERS=',MNLAY
      WRITE(*,*)
C
C        DIS FILE PROMPT
C
       GO TO 731
  730   WRITE(*,*)' PROBLEM FINDING DIS FILE, TRY AGAIN!'
  731   WRITE (*,*)' ENTER INPUT DISCRETIZATION FILE NAME :'
      READ(*,30) DNAME
C
      INQUIRE(FILE=DNAME,EXIST=FILEFOUND)
      IF(.NOT.FILEFOUND) THEN
            WRITE(*,*)'CAN NOT FIND FILE: ',DNAME
            DNAME=''
            GO TO 731
            ENDIF
C
      WRITE(*,*) ' INPUT NAME IS: ', DNAME
C
   30 FORMAT(A20)
      OPEN(UNIT=INZN6,FILE=DNAME,STATUS='OLD',ERR=730,FORM='FORMATTED')
      READ(INZN6,*) C
C
      IF(C.NE.'#')THEN
      BACKSPACE(INZN6)
      READ(INZN6,*) NLAY,NROW,NCOL
      ELSE
      READ(INZN6,*) C
       IF(C.NE.'#')THEN
       BACKSPACE(INZN6)
       READ(INZN6,*) NLAY,NROW,NCOL
       ELSE
       READ(INZN6,*) NLAY,NROW,NCOL
       ENDIF
      ENDIF
C
      WRITE(*,*) 'MODEL LAYERS=',NLAY,'ROWS=',NROW,'COLS=',NCOL
C
C         CHECK FILE PROMPT
C
      GO TO 751
  750  WRITE(*,*)' BAD RESPONSE, TRY AGAIN! '
  751  WRITE(*,*)
       WRITE(*,*)' ENTER OUTPUT CHECK FILE TO WRITE CONC. ARRAY: '
      READ(*,40) CNAME
   40 FORMAT(A20)
C
            IF(CNAME.EQ.' ') THEN
            GO TO 751
            ENDIF
C
      OPEN(UNIT=INZN5,FILE=CNAME,STATUS='REPLACE',ERR=750,
     1FORM='FORMATTED')
       WRITE(INZN5,*)'CHECK FILE FOR: ',ONAME
C
C-----------------------------------------------------------
C            READ ZONE FILE OR SPECIFY ZONES
C
C-----OPEN THE ZONE FILE IF IT EXISTS
      WRITE(*,*)
      WRITE(*,*) ' Enter the name of ZONE INPUT FILE(cr=INTERACTIVE:'
C
      READ(*,'(A)')ZNAME
C
      IF(ZNAME.EQ.' ')THEN
C            IF ZNAME IS BLANK
      CALL BLOCK(IZONE,NLAY,NROW,NCOL,NZDIM,NZONES,IOUT)
C
      ELSE
C        WHEN ZNAME IS NOT BLANK
        GO TO 761
  760  WRITE(*,*)' PROBLEM FINDING ZONE FILE, TRY AGAIN! '
  761  OPEN(UNIT=INZN1,FILE=ZNAME,STATUS='OLD',ERR=760,FORM='FORMATTED')
      WRITE(*,*) 'ZONE FILE NAME:',ZNAME
C-----WHEN NAME IS NOT BLANK, OPEN ZONE FILE, AND CHECK GRID DIMENSIONS
         WRITE(IOUT,*)
         WRITE(IOUT,*) ' The zone file is:'
         WRITE(IOUT,*) ZNAME
         READ(INZN1,*) NL,NR,NC
         WRITE(IOUT,*) NL,NR,NC
         IF(NC.NE.NCOL .OR. NR.NE.NROW .OR. NL.NE.NLAY) THEN
            WRITE(*,*) 'MISMATCH BETWEEN DIMENSIONS OF DISCRET. DATA
     1 AND ZONE DATA:'
            WRITE(*,*) 'LAYERS, ROWS, COLUMNS IN ZONE DATA:',NL,NR,NC
            WRITE(*,*) 'LAYERS, ROWS, COLUMNS IN DISCRET. FILE:',
     1                  NLAY,NROW,NCOL
            STOP
         END IF
C
C-----READ ZONE ARRAY
      CALL IZREAD(IZONE,NROW,NCOL,NLAY,NZDIM,INZN1,INZN2,IOUT,NZONES)
C
C----CLOSE ZONE FILE
         CLOSE(UNIT=INZN1)
      ENDIF
C
      WRITE(IOUT,*)'NUMBER OF ZONES: ',NZONES
C
C------------------------------------------------------------
C         READ CONCENTRATION FILE, PART 1
C         (11 statement is referenced for errors reading header)
C         (first time through, file is not rewound, second it is)
C
C              ERROR COUNTER
   11   IERR=IERR+1
      IF(IERR.EQ.2)REWIND(15)
C
C------------------------------------------------------------
C           READ CONCENTRATION FILE,PART 2
C           AND PRINT HEADINGS FOR OUTPUT FILE
C
      WRITE(IOUT,*)'READING CONCENTRATION DATA'
       WRITE(IOUT,*)'-------------------------------'
       WRITE(IOUT,502)
 502   FORMAT('STATISTICS ARE CALCULATED ON NON-NEGATIVE (VALUES=> 0)
     1CONCENTRATION EXCEPT PCOUNT (VALUES > 0)')
       WRITE(IOUT,*)
          WRITE(IOUT,504)
 504   FORMAT(2X,'TIME',7X,'ZONE',3X,'MAX.',6X,'ROW',2X,
     1'COL.',1X,'LAYER',2X,'MIN.',6X,'MEAN',7X,'TOTAL',6X,'COUNT',3X,
     2'PCOUNT',4X,'STAND.DEV.')
C
C------------------------------------------------------------
C          READ HEADER OF CONCENTRATION FILE
C
 97   READ(INZN3,31,ERR=99,END=99)HEADER
 31    FORMAT(A14)
C
       ICOUNT=ICOUNT+1
C
          IF(HEADER.EQ.'CONCENTRATIONS')THEN
C
          BACKSPACE(15)
          READ(INZN3,32)HEADER,SUMTCH2
 32       FORMAT(A14,76X,G10.4)
C
C
               WRITE(*,*)'TIME=',SUMTCH2,',','# OF Headers read',ICOUNT
               BACKSPACE(15)
               GO TO 96
               ELSE
               GO TO 97
               ENDIF
C----------------------------------------------------
C              INITIALIZE MODEL SUBSCRIPTS
C 96           L=0
C              J=0
C              I=0
C              LL=0
C              JJ=0
C              II=0
C-----------------------------------------------------
C            READ CONCENTRATION ARRAY AND PERFORM STATS
C
  96     CALL RARRAY(IZONE,ARRAY,NROW,NCOL,NLAY,MNROW,MNCOL,MNLAY,
     1NZDIM,INZN3,INZN4,INZN5,INZN6,INZN1,INZN2,IOUT,NZONES,
     2LLAYER,ONAME,SUMTCH2,CMAX,CMIN,NCOUNT,NCOUNTP,CDIFF,CSTD,CTOTAL,
     3CAVG,IROW,JCOL,JMAX,IMAX,LMAX,IICOUNT,ICOUNT)
C---------------------------------------------------------
C

C          READ ANOTHER HEADER
      GO TO 97
C
C *** DONE ***
   99 CONTINUE
C
C
      WRITE(*,*)IICOUNT,'=NUMBER OF ACTIVE CONC. CELLS COUNTED'
C
      STOP
      END
C////////////////////////////////////////////////////////////////////////
C
      SUBROUTINE BLOCK(IZONE,NLAY,NROW,NCOL,NZDIM,NZONES,IOUT)
C     ******************************************************************
C     INPUT ZONE VALUES INTERACTIVELY BY BLOCK (IZONE)
C     IOUT IS MAIN OUTPUT FILE
C     -----------------------------
C       FROM ZONEBUDGET VERSION 1
C       By Arlen W. Harbaugh, December 20, 1996
C       Modified by Philip T. Harte, May 2005
C     ******************************************************************
      DIMENSION IZONE(NCOL,NROW,NLAY)
C     ------------------------------------------------------------------
C
C-----INITIALIZE THE IZONE ARRAY
      DO 5 L=1,NLAY
      DO 5 I=1,NROW
      DO 5 J=1,NCOL
      IZONE(J,I,L)=0
5     CONTINUE
      WRITE(IOUT,*)
C
10    WRITE(*,*)
      WRITE(*,*) ' Enter the start layer, stop layer (0,0 means done):'
      READ(*,*) L1,L2
      IF(L1.EQ.0 .AND. L2.EQ.0) RETURN
      IF(L1.LT.1 .OR. L2.GT.NLAY) THEN
         WRITE(*,*) ' NON-EXISTENT LAYER -- TRY AGAIN'
         GO TO 10
      END IF
20    WRITE(*,*) ' Enter the start row, stop row:'
      READ(*,*) I1,I2
      IF(I1.LT.1 .OR. I2.GT.NROW) THEN
         WRITE(*,*) ' NON-EXISTENT ROW -- TRY AGAIN'
         GO TO 20
      END IF
30    WRITE(*,*) ' Enter the start column, stop column:'
      READ(*,*) J1,J2
      IF(J1.LT.1 .OR. J2.GT.NCOL) THEN
         WRITE(*,*) ' NON-EXISTENT COLUMN -- TRY AGAIN'
      END IF
C
40    WRITE(*,*) 'Enter the zone for this block,must no. incremental:'
      READ(*,*) ICONST
      IF(ICONST.LT.0) THEN
         WRITE(*,*) ' NEGATIVE ZONES ARE NOT ALLOWED'
         GO TO 40
      END IF
      IF(ICONST.GT.NZDIM) THEN
         WRITE(*,*) ' THE HIGHEST ZONE ALLOWED IS',NZDIM
         GO TO 40
      END IF
      IF(ICONST.GT.NZONES) NZONES=ICONST
C
      WRITE(IOUT,3) L1,L2,I1,I2,J1,J2,ICONST
3     FORMAT(1X,'Zone block: LAYERS ',I2,'-',I2,
     1                  '      ROWS ',I3,'-',I3,
     2               '      COLUMNS ',I3,'-',I3,
     3                 '      VALUE:',I3)
C
      DO 50 L=L1,L2
      DO 50 I=I1,I2
      DO 50 J=J1,J2
      IZONE(J,I,L)=ICONST
50    CONTINUE
      GO TO 10
C
      END
C/////////////////////////////////////////////////////////////////////////
      SUBROUTINE IZREAD(IZONE,NROW,NCOL,NLAY,NZDIM,INZN1,INZN2,IOUT,
     1                  NZONES)
C     ******************************************************************
C     ROUTINE TO READ A 3-D ZONE MATRIX FILE (IZONE)
C       INZN1 IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C
C       FROM ZONEBUDGET VERSION 1
C       By Arlen W. Harbaugh, December 20, 1996
C       Modified by Philip T. Harte, May 2005
C     ******************************************************************
C        SPECIFICATIONS:
      DIMENSION IZONE(NCOL,NROW,NLAY)
      CHARACTER*20 FMTIN
      CHARACTER*80 NAME,NAMPRV
      CHARACTER*10 LOCAT,LOCTMP
C     ------------------------------------------------------------------
      WRITE(IOUT,*) 'READING ZONE DATA'
      NAMPRV=' '
      DO 1000 L=1,NLAY
C
C-----READ ARRAY CONTROL RECORD.
      READ (INZN1,201) LOCAT,ICONST,FMTIN,IPRN
201     FORMAT(A,I10,A,I10)
        WRITE(IOUT,*)LOCAT
C
C-----LEFT JUSTIFY LOCAT AND MAKE IT UPPER CASE SO USER DOES NOT HAVE TO
C-----BE CONCERNED WITH JUSTIFICATION OR CASE
      LOCTMP=LOCAT
      IDIFF=ICHAR('a')-ICHAR('A')
      DO 200 I=1,10
      IF(LOCTMP(I:I).GE.'a' .AND. LOCTMP(I:I).LE.'z')
     1        LOCTMP(I:I)=CHAR(ICHAR(LOCTMP(I:I))-IDIFF)
200    CONTINUE
      DO 230 I=1,10
      IF(LOCTMP(I:I).NE.' ') GO TO 240
230    CONTINUE
      I=1
240    LOCAT=LOCTMP(I:10)
C
C-----USE LOCAT TO SEE WHERE ARRAY VALUES COME FROM.
      IF(LOCAT.NE.'CONSTANT') GO TO 290
C
C-----LOCAT='CONSTANT' -- SET ALL ARRAY VALUES EQUAL TO ICONST.
      DO 280 I=1,NROW
      DO 280 J=1,NCOL
280    IZONE(J,I,L)=ICONST
      WRITE(IOUT,*)
      WRITE(IOUT,283)ICONST,L
283    FORMAT(13X,'Zone Array =',I3,' for layer',I3)
      IF(ICONST.GT.NZONES) NZONES=ICONST
      IF(ICONST.LT.0) THEN
         WRITE(*,*) ' NEGATIVE ZONE NUMBER IS NOT ALLOWED'
         STOP
      END IF
      GO TO 1000
C
C-----LOCAT SHOULD BE EITHER 'EXTERNAL' OR 'INTERNAL'
C-----IF 'INTERNAL', READ ARRAY FROM SAME FILE
290    IF(LOCAT.EQ.'INTERNAL') THEN
         INUNIT=INZN1
         WRITE(IOUT,*)
         WRITE(IOUT,291) L
291       FORMAT(1X,'Zone Array for layer',I3,
     1      ' will be read from the Zone File')
C
C-----IF 'EXTERNAL', OPEN A SEPARATE FILE
      ELSE IF(LOCAT.EQ.'EXTERNAL') THEN
         READ(INZN1,'(A)') NAME
         INUNIT=INZN2
         WRITE(IOUT,*)
         WRITE(IOUT,293) L,NAME
293       FORMAT(1X,'Zone Array for layer',I3,
     1         ' will be read from file:'/1X,A)
         IF(NAME.NE.NAMPRV) THEN
            IF(NAMPRV.NE.' ') CLOSE(UNIT=INUNIT)
            OPEN(UNIT=INUNIT,FILE=NAME,STATUS='OLD')
            WRITE(IOUT,296)
296          FORMAT(1X,'The file was opened successfully.')
            NAMPRV=NAME
         ELSE
            WRITE(IOUT,297)
297          FORMAT(1X,'This file is already open -- will continue readi
     1ng from the current location.')
         END IF
C
C-----LOCAT IS INVALID
      ELSE
         WRITE(*,*) ' INVALID LOCAT IN ARRAY CONTROL RECORD:',LOCAT
         STOP
      END IF
C
C-----LOCAT>0 -- READ RECORDS USING FREE-FORMAT OR FMTIN.
      IF(FMTIN.EQ.' ') THEN
         WRITE(IOUT,298) L
298       FORMAT(1X,'Zone Array for layer',I3,
     1       ' will be read using free format.'/1X,55('-'))
         DO 210 I=1,NROW
         READ(INUNIT,*) (IZONE(J,I,L),J=1,NCOL)
210      CONTINUE
      ELSE
         WRITE(IOUT,204) L,FMTIN
204      FORMAT(1X,'Zone Array for layer',I3,
     1       ' will be read using format: ',A/1X,71('-'))
         DO 410 I=1,NROW
         READ (INUNIT,FMTIN) (IZONE(J,I,L),J=1,NCOL)
410      CONTINUE
      END IF
C
C-----FIND THE MAXIMUM ZONE AND CHECK FOR NEGATIVE IZONE VALUES
320   DO 400 I=1,NROW
      DO 400 J=1,NCOL
      IF(IZONE(J,I,L).LT.0) THEN
         WRITE(*,*) ' NEGATIVE ZONE AT (LAYER,ROW,COLUMN):',L,I,J
         STOP
      END IF
      IF(IZONE(J,I,L).GT.NZONES) NZONES=IZONE(J,I,L)
400   CONTINUE
C
C-----IF PRINT CODE (IPRN) =>0 THEN PRINT ARRAY VALUES.
      IF(IPRN.LT.0) GO TO 1000
C
C-----PRINT COLUMN NUMBERS AT THE TOP OF THE PAGE
      WRITE(IOUT,421) (I,I=1,NCOL)
421   FORMAT(/,(5X,25I3))
      WRITE(IOUT,422)

422   FORMAT(1X,79('-'))
C
C-----PRINT EACH ROW IN THE ARRAY.
      DO 430 I=1,NROW
      WRITE(IOUT,423) I,(IZONE(J,I,L),J=1,NCOL)
423   FORMAT(1H0,I3,1X,25I3/(5X,25I3))
430   CONTINUE
C
1000  CONTINUE
      IF(NAMPRV.NE.' ') CLOSE(UNIT=INZN2)
C
C-----CHECK FOR TOO MANY ZONES
      IF(NZONES.GT.NZDIM) THEN
         WRITE(*,*) ' YOUR HIGHEST ZONE EXCEEDS THE PROGRAM MAXIMUM'
         WRITE(*,*) NZDIM,' IS THE HIGHEST ZONE ALLOWED'
         WRITE(*,*) NZONES,' IS YOUR HIGHEST ZONE'
         STOP
      END IF
C
C-----RETURN
      RETURN
      END
C/////////////////////////////////////////////////////////////////////////
      SUBROUTINE RARRAY(IZONE,ARRAY,NROW,NCOL,NLAY,MNROW,MNCOL,MNLAY,
     1NZDIM,INZN3,INZN4,INZN5,INZN6,INZN1,INZN2,IOUT,NZONES,
     2LLAYER,ONAME,SUMTCH2,CMAX,CMIN,NCOUNT,NCOUNTP,CDIFF,CSTD,CTOTAL,
     3CAVG,IROW,JCOL,JMAX,IMAX,LMAX,IICOUNT,ICOUNT)
C     ******************************************************************
C     ROUTINE TO READ CONCENTRATION ARRAY, TABULATE STATS, AND PRINT IZONE
C      INZN3 IS INPUT UNIT CONCENTRATION ARRRAY
C      IOUT IS OUTPUT UNIT MAIN OUTPUT FILE, INZN5 OUTPUT UNIT FILE 2
C     ******************************************************************
C       DIMENSIONS:
      DIMENSION IZONE(NCOL,NROW,NLAY)
      DIMENSION ARRAY(MNCOL,MNROW,MNLAY)
      DIMENSION CMAX(NZDIM)
      DIMENSION CMIN(NZDIM)
      DIMENSION CTOTAL(NZDIM)
      DIMENSION NCOUNT(NZDIM)
      DIMENSION NCOUNTP(NZDIM)
      DIMENSION CDIFF(NZDIM)
      DIMENSION CSTD(NZDIM)
      DIMENSION CAVG(NZDIM)
      DIMENSION IMAX(NZDIM)
      DIMENSION JMAX(NZDIM)
      DIMENSION LMAX(NZDIM)
      DIMENSION IMIN(NZDIM)
      DIMENSION JMIN(NZDIM)
      DIMENSION LMIN(NZDIM)
C
C      SPECIFICATIONS:
C
      CHARACTER*20 ANAME,ONAME,HEADER,ZNAME,MNAME,CNAME,DNAME
      DOUBLE PRECISION CMAX,CAVG,CTOTAL,CDIFF,CSTD
      DOUBLE PRECISION ARRAY,CMIN,SUMTCH2
C      REAL*16 ARRAY,CMIN
C------------------------------------------------------------------
C          INITIALIZE ZONE VALUES
C
          DO 50 LZ=1,NZDIM
          CMAX(LZ)=0.
          CMIN(LZ)=1.000E+16
          NCOUNT(LZ)=0.
          NCOUNTP(LZ)=0.
          CDIFF(LZ)=0.
          CSTD(LZ)=0.
          CTOTAL(LZ)=0.
          CAVG(LZ)=0.
   50     CONTINUE
C
C------------------------------------------------------------------
C                 READ  LAYERS
C
            DO 110 LL=1,MNLAY
C               RELATION MODEL GRID TO SUBGRID
             L=LL+(LLAYER-1)
C
C              SKIP LINE
          READ(INZN3,*,ERR=98,END=98)HEADER
C
         WRITE(INZN5,*)
        WRITE(INZN5,*)HEADER,'LAYER=',L,'SLAYER=',LL,'TIME=',SUMTCH2
          WRITE(INZN5,*)'MNCOL=',MNCOL,'MNROW=',MNROW
C/////////////////SKiP FOR VERSION 1.3+   /////////////////
C          INITIALIZE ZONE VALUES PER LAYER
C
C          DO 50 LZ=1,NZDIM
C          CMAX(LZ)=0.
C          CMIN(LZ)=0.
C          NCOUNT(LZ)=0.
C          NCOUNTP(LZ)=0.
C          CDIFF(LZ)=0.
C          CSTD(LZ)=0.
C          CTOTAL(LZ)=0.
C          CAVG(LZ)=0.
C   50     CONTINUE
C////////////////////////////////////////////////////////
C
C              READ ROWS (READS ENTIRE ROW ALL CELLS)
C
        DO 90 II=1,MNROW
           I=II+(IROW-1)
C
C           WRITE(*,*)'ROW=',I,'SROW=',II
C
         READ(INZN3,*,ERR=92)(ARRAY(JJ,II,LL),JJ=1,MNCOL)
C
C////////////////////////////////////////////////////////////
C        OPTIONAL CHECK HERE
C      Only prints first 600 cells of last row of concentration array
C
C
          IF(II.EQ.MNROW)THEN
          IF(LL.EQ.MNLAY)WRITE(INZN5,91)(ARRAY(JJ,II,LL),JJ=1,MNCOL)
  91      FORMAT(60(10G12.4))
           ELSE
           ENDIF
C
C//////////////////////////////////////////////////////////
        GO TO 93
   92  WRITE(IOUT,*)' ERROR IN ROW/COLUMN LENGTH TRY AGAIN OR KILL.'
        GO TO 98
C/////////////////////////////////////////////////////////
C          SET ALL CELLS (COLUMNS) PER ROW where
C       LZ= IBOUND AND SET J OF GRID AND JJ OF SUBGRID
C
   93        DO 94 JJ=1,MNCOL
             J=JJ+(JCOL-1)
             LZ=IZONE(J,I,L)
C
C
C              IGNORE NEGATIVE CONCENTRATIONS
            IF(ARRAY(JJ,II,LL).LT.0E+00)GO TO 117
C
C                DO NOT TABULATE FOR ZONE = 0
              IF(LZ.EQ.0)GO TO 117
C---------------------------------------------------------
C           TABULATE STATISTICS
C
           IICOUNT=IICOUNT+1
           NCOUNT(LZ)=NCOUNT(LZ)+1.
           IF(ARRAY(JJ,II,LL).GT.0)THEN
           NCOUNTP(LZ)=NCOUNTP(LZ)+1
           ELSE
           ENDIF
           CTOTAL(LZ)=CTOTAL(LZ)+ARRAY(JJ,II,LL)
           CAVG(LZ)=CTOTAL(LZ)/NCOUNT(LZ)
           CDIFF(LZ)=CDIFF(LZ)+((ARRAY(JJ,II,LL)-CAVG(LZ))*
     1(ARRAY(JJ,II,LL)-CAVG(LZ)))
           CSTD(LZ)=SQRT((CDIFF(LZ))/(NCOUNT(LZ)))
C
C          CALCULATE MAXIMUM AND LOCATION
C
            IF(ARRAY(JJ,II,LL).LT.CMAX(LZ))GO TO 95
            JMAX(LZ)=J
            IMAX(LZ)=I
            LMAX(LZ)=L
            CMAX(LZ)=MAX(CMAX(LZ),ARRAY(JJ,II,LL))
C
C            CALCULATE MIN
   95           CMIN(LZ)=MIN(CMIN(LZ),ARRAY(JJ,II,LL))
C
C            PRINTING MINIMUM LOCATION NOT USED
C                JMIN(LZ)=J
C                IMIN(LZ)=I
C                LMIN(LZ)=L
C
C---------------------------------------------------------
C          OPTIONAL CHECK HERE
C        IF(JJ.EQ.MNCOL)THEN
C       WRITE(INZN5,*)'L',' ZONE',' ROW',' SROW',' COL',' SCOL', ' CT'
C        WRITE(INZN5,*)L,LZ,I,II,J,JJ,CTOTAL(LZ)
C        ELSE
C        ENDIF
C---------------------------------------------------------
C
C
 117          IF(II.EQ.MNROW)THEN
                 IF(JJ.EQ.MNCOL)THEN
                 GO TO 118
                 ELSE
                 GO TO 94
                 ENDIF
              ELSE
              GO TO 94
              ENDIF
C
C
C--------------------------------------------------------
C          PRINT STATISTICS PER ZONE
C
 118        DO 120 LZ=1,NZONES
C
             IF(NCOUNT(LZ).GT.0)THEN
             IF(LL.EQ.MNLAY) THEN
C
             WRITE(INZN5,*)'PRINT STATS FOR=',I,II,J,JJ,'zone ',LZ
             WRITE(INZN5,*)
C
C            IF(IZONE(J,I,L).GT.0)THEN
C
      WRITE(IOUT,503)SUMTCH2,LZ,CMAX(LZ),IMAX(LZ),JMAX(LZ),LMAX(LZ),
     1CMIN(LZ),CAVG(LZ),CTOTAL(LZ),NCOUNT(LZ),NCOUNTP(LZ),CSTD(LZ)
 503  FORMAT(G12.6,3X,I3,1X,E11.4,1X,I4,1X,I4,1X,I3,1X,2E11.4,E11.4,1X,
     1I7,1X,I7,2X,E11.4)
             ELSE
             ENDIF
C
         ELSE
         ENDIF
C
 120  CONTINUE
  94  CONTINUE
  90  CONTINUE
 110  CONTINUE
C---------------------------------------------------------
C             PRINT OUT IZONES
C           Only prints first 600 cells
C
        IF(ICOUNT.EQ.1)THEN
        WRITE(INZN5,*)'ZONES OF MODEL GRID'
C
C
        DO 700 L=1,NLAY
        WRITE(INZN5,*)'IZONE LAYER=',L
        DO 702 I=1,NROW
        WRITE(INZN5,701)(IZONE(J,I,L),J=1,NCOL)
 701    FORMAT(60(10I3))
 702    CONTINUE
 700    CONTINUE
        ELSE
        ENDIF
C---------------------------------------------------------
C
C
C-----RETURN
  98  RETURN
      END

